﻿Option Strict On

''' <summary>
''' LockManager - Sperren/Entsperren von Datensätzen einer MySQL Datenbank
''' Author: Gerhard Wenzl
''' Date: 22.03.2021
''' 
''' 
''' 
''' </summary>
Public NotInheritable Class LockManager
    Private Shared m_ErrMsg As String = ""
    Private Shared m_Connection As MySql.Data.MySqlClient.MySqlConnection
    Private Shared m_Lock As LockInfo
    Private Shared m_LockedInfo As LockInfo
    Private Shared m_TimeOut As Integer = 5 ' 5 Minuten Timeout für Datensatz sperren
    Private Shared m_SessionId As String = Guid.NewGuid().ToString()
    Private Shared m_InitFlag As Boolean = False

    ''' <summary>
    ''' MySQLConnection zur Datenbank
    ''' Beim Setzen der MySqlConnection wird geprüft, ob die LockManager Tabelle vorhanden ist 
    ''' und es wird ein Property InitFlag gesetzt
    ''' </summary>
    ''' <returns>aktuell gesetzte MySqlConnection</returns>
    Shared Property Connection As MySql.Data.MySqlClient.MySqlConnection
        Get
            Return m_Connection
        End Get
        Set(value As MySql.Data.MySqlClient.MySqlConnection)
            If value IsNot Nothing Then
                m_Connection = value
                ' Prüfen, ob LockManager Tabelle vorhanden ist.
                If Not CheckAndCreateTable() Then
                    MsgBox("Konnte LockManager Tabelle nicht erzeugen!")
                    m_InitFlag = False
                Else
                    m_InitFlag = True
                End If
            Else
                MsgBox("Keine gültige Connection angegeben!")
                m_InitFlag = False
            End If
        End Set
    End Property

    ''' <summary>
    ''' Property InitFlag: Zeigt an, ob die Connection und auch die Tabelle LockManager vorhanden sind
    ''' </summary>
    ''' <returns>True/False</returns>
    Shared ReadOnly Property InitFlag() As Boolean
        Get
            Return m_InitFlag
        End Get
    End Property

    ''' <summary>
    ''' Property UserName: Name des Benutzers
    ''' </summary>
    ''' <returns>Name des Benutzers</returns>
    Shared Property UserName As String = ""

    ''' <summary>
    ''' Property SessionId: GUID für eindeutige Identifikation
    ''' </summary>
    ''' <returns>SessioId</returns>
    Shared ReadOnly Property SessionId As String
        Get
            Return m_SessionId
        End Get
    End Property

    ''' <summary>
    ''' Property LockedInfo: Lock Informationen, wer den Datensatz gesperrt hat
    ''' </summary>
    ''' <returns>Lock Informationen</returns>
    Shared ReadOnly Property LockedInfo() As LockInfo
        Get
            Return m_LockedInfo
        End Get
    End Property

    ''' <summary>
    ''' Property ErrorMsg: Error Meldungen oder Hinweise
    ''' </summary>
    ''' <returns>Error Meldungen oder Hinweise</returns>
    Shared ReadOnly Property ErrorMsg As String
        Get
            Return m_ErrMsg
        End Get
    End Property

    ''' <summary>
    ''' Propert GetFieldNameOfPrimaryKey: Tabellenspaltenname des PrimaryKey und AutoIncrement
    ''' </summary>
    ''' <param name="row">Datensatz</param>
    ''' <returns>Name der Tabellenspalte</returns>
    Shared ReadOnly Property GetFieldNameOfPrimaryKey(row As DataRow) As String
        Get
            Return GetPrimaryKeyFieldName(row)
        End Get
    End Property

    Private Sub New()
        ' Nix instanzieren
    End Sub

    ''' <summary>
    ''' Sperrt einen Datensatz und gibt eine Lock Information zurück
    ''' </summary>
    ''' <param name="row">zu sperrende Datensatz</param>
    ''' <returns>Lock Informationen</returns>
    Overloads Shared Function Lock(row As DataRow) As LockInfo
        m_ErrMsg = ""
        m_Lock = New LockInfo()
        ' Prüfe, ob Initialisierungsflag gesetzt ist - d.h. eine MySqlConnection und die LockManager Tabelle vorhanden sind
        If Not InitFlag Then Return m_Lock
        ' Lockinformationen zuweisen - Infos vom Datensatz holen 
        m_Lock.TableName = row.Table.TableName
        Dim m_RecordIdField As String = GetPrimaryKeyFieldName(row)
        If m_RecordIdField = "" Then
            m_ErrMsg = "Kein PrimaryKey Field im DataRow mit AutoIncrement gefunden!"
            m_Lock.Success = False
            Return m_Lock
        End If
        m_Lock.RecordID = CInt(row(m_RecordIdField))
        ' aktuelle Server DateTime lesen
        m_Lock.LockTimeout = GetServerDateTime(True)
        If m_Lock.LockTimeout Is Nothing Then
            m_Lock.Success = False
            Return m_Lock
        End If
        ' Versuche den Datensatz zu sperren 
        If TryLock() Then Return m_Lock
        m_Lock.Success = False
        Return m_Lock
    End Function
    ''' <summary>
    ''' Sperrt einen Datensatz und gibt eine Lock Information zurück
    ''' </summary>
    ''' <param name="pTableName">Tabellenname</param>
    ''' <param name="pRecordId">Id des zu sperrenden Datensatzes</param>
    ''' <returns>Lock Informationen</returns>
    Overloads Shared Function Lock(pTableName As String, pRecordId As Integer) As LockInfo
        m_ErrMsg = ""
        m_Lock = New LockInfo()
        ' Prüfe, ob Initialisierungsflag gesetzt ist - d.h. eine MySqlConnection und die LockManager Tabelle vorhanden sind
        If Not InitFlag Then Return m_Lock
        ' Lockinformationen zuweisen
        m_Lock.TableName = pTableName
        m_Lock.RecordID = pRecordId
        ' aktuelle Server DateTime lesen
        m_Lock.LockTimeout = GetServerDateTime(True)
        If m_Lock.LockTimeout Is Nothing Then
            m_Lock.Success = False
            Return m_Lock
        End If
        ' Versuche den Datensatz zu sperren 
        If TryLock() Then Return m_Lock
        m_Lock.Success = False
        Return m_Lock
    End Function

    ''' <summary>
    ''' Prüft, ob die Sperrung noch gültig ist
    ''' </summary>
    ''' <returns>True/False</returns>
    Overloads Shared Function CheckLock() As Boolean
        m_ErrMsg = ""
        ' Prüfe, ob Initialisierungsflag gesetzt ist - d.h. eine MySqlConnection und die LockManager Tabelle vorhanden sind
        If Not InitFlag Then Return False
        If m_Lock Is Nothing Then Return False
        ' Infos von m_Lock verwenden
        ' Prüfe, ob der RecordLock noch gültig ist
        If Not TryCheckLock() Then Return False
        Return True
    End Function
    ''' <summary>
    ''' Prüft, ob die Sperrung noch gültig ist
    ''' </summary>
    ''' <param name="pLockInfo">Lock Informationen eines gespeicherten Datensatzes</param>
    ''' <returns>True/False</returns>
    Overloads Shared Function CheckLock(pLockInfo As LockInfo) As Boolean
        m_ErrMsg = ""
        ' Prüfe, ob Initialisierungsflag gesetzt ist - d.h. eine MySqlConnection und die LockManager Tabelle vorhanden sind
        If Not InitFlag Then Return False
        If pLockInfo Is Nothing Then Return False
        ' Infos von pLockInfo zuweisen
        m_Lock = pLockInfo
        ' Prüfe, ob der RecordLock noch gültig ist
        If Not TryCheckLock() Then Return False
        Return True
    End Function
    ''' <summary>
    ''' Prüft, ob die Sperrung noch gültig ist
    ''' </summary>
    ''' <param name="pLockName">LockName eines gespeicherten Datensatzes, welcher in der MemoryKlasse hinterlegt ist</param>
    ''' <returns>True/False</returns>
    Overloads Shared Function CheckLock(pLockName As String) As Boolean
        m_ErrMsg = ""
        ' Prüfe, ob Initialisierungsflag gesetzt ist - d.h. eine MySqlConnection und die LockManager Tabelle vorhanden sind
        If Not InitFlag Then Return False
        Dim retValue As Object = MemoryHelper.Item(pLockName)
        If retValue Is Nothing Then
            m_ErrMsg = "LockName nicht in der MemoryKlasse gefunden!"
            Return False
        End If
        ' Infos von retValue zuweisen
        m_Lock = CType(retValue, LockInfo)
        ' Prüfe, ob der RecordLock noch gültig ist
        If Not TryCheckLock() Then Return False
        Return True
    End Function

    ''' <summary>
    ''' Gibt den gesperrten Datensatz wieder frei
    ''' </summary>
    ''' <returns>True/False</returns>
    Overloads Shared Function Unlock() As Boolean
        m_ErrMsg = ""
        ' Prüfe, ob Initialisierungsflag gesetzt ist - d.h. eine MySqlConnection und die LockManager Tabelle vorhanden sind
        If Not InitFlag Then Return False
        If m_Lock Is Nothing Then Return True
        ' Infos von m_Lock verwenden
        ' Datensatz wieder freigeben d.h. aus LockManager Tabelle löschen
        If Not Delete() Then Return False
        Return True
    End Function
    ''' <summary>
    ''' Gibt den gesperrten Datensatz wieder frei
    ''' </summary>
    ''' <param name="pLockInfo">Lock Informationen eines gespeicherten Datensatzes</param>
    ''' <returns>True/False</returns>
    Overloads Shared Function Unlock(pLockInfo As LockInfo) As Boolean
        m_ErrMsg = ""
        ' Prüfe, ob Initialisierungsflag gesetzt ist - d.h. eine MySqlConnection und die LockManager Tabelle vorhanden sind
        If Not InitFlag Then Return False
        If pLockInfo Is Nothing Then Return True
        ' Infos von pLockInfo zuweisen
        m_Lock = pLockInfo
        ' Datensatz wieder freigeben d.h. aus LockManager Tabelle löschen
        If Not Delete() Then Return False
        Return True
    End Function
    ''' <summary>
    ''' Gibt den gesperrten Datensatz wieder frei
    ''' </summary>
    ''' <param name="pLockName">LockName eines gespeicherten Datensatzes, welcher in der MemoryKlasse hinterlegt ist</param>
    ''' <returns>True/False</returns>
    Overloads Shared Function Unlock(pLockName As String) As Boolean
        m_ErrMsg = ""
        ' Prüfe, ob Initialisierungsflag gesetzt ist - d.h. eine MySqlConnection und die LockManager Tabelle vorhanden sind
        If Not InitFlag Then Return False
        Dim retValue As Object = MemoryHelper.Item(pLockName)
        If retValue Is Nothing Then
            m_ErrMsg = "LockName nicht in der MemoryKlasse gefunden!"
            Return False
        End If
        ' Infos von retValue zuweisen
        m_Lock = CType(retValue, LockInfo)
        ' Datensatz wieder freigeben d.h. aus LockManager Tabelle löschen
        If Not Delete() Then Return False
        Return True
    End Function

    ''' <summary>
    ''' Liefert ein aktuelles DateTime vom Server im Format yyyy-MM-dd HH:mm:ss
    ''' </summary>
    ''' <param name="AsLockTimeOut">Flag, ob zur aktuellen DateTime des Servers die LockTimeout addiert werden soll</param>
    ''' <returns>String im Format [yyyy-MM-dd HH:mm:ss]</returns>
    Shared Function GetServerDateTime(Optional AsLockTimeOut As Boolean = False) As String
        Try
            m_ErrMsg = ""
            Dim retValue As String = Nothing
            Dim dbSQL As String = ""
            ' Flag, ob die LockTimeout ermittelt werden soll, oder nur die aktuelle Server DateTime
            If AsLockTimeOut Then
                dbSQL = "Select DATE_ADD(Now(), INTERVAL " + m_TimeOut.ToString + " MINUTE) as LockTimeOut;"
            Else
                dbSQL = "Select Now() as ServerDateTime;"
            End If
            If Connection.State <> ConnectionState.Closed Then
                Connection.Close()
            End If
            Connection.Open()

            Using myCommand As MySql.Data.MySqlClient.MySqlCommand = New MySql.Data.MySqlClient.MySqlCommand(dbSQL, Connection)
                Using myReader As MySql.Data.MySqlClient.MySqlDataReader = myCommand.ExecuteReader()
                    Try
                        While myReader.Read()
                            retValue = myReader.GetString(0)
                        End While
                    Catch ex As Exception
                        m_ErrMsg = ex.Message
                    End Try
                End Using
            End Using
            Connection.Close()
            ' In ein MySQL DateTime Format umwandeln
            retValue = CDate(retValue).ToString("yyyy-MM-dd HH:mm:ss")
            Return retValue
        Catch ex As Exception
            If Connection.State <> ConnectionState.Closed Then
                Connection.Close()
            End If
            Return Nothing
        End Try
    End Function

    ''' <summary>
    ''' Prüft, ob der Datensatz zu den LockInformationen gehört
    ''' Verglichen wird Tabellenname und RecordId
    ''' </summary>
    ''' <param name="row">Datensatz</param>
    ''' <param name="pLockInfo">LockInformationen</param>
    ''' <returns>True/False</returns>
    Overloads Shared Function CompareDataRowWithLockInfo(row As DataRow, pLockInfo As LockInfo) As Boolean
        If pLockInfo Is Nothing Then Return False
        If row Is Nothing Then Return False
        Return Compare(row, pLockInfo)
    End Function
    ''' <summary>
    ''' Prüft, ob der Datensatz zu den internen LockInformationen (m_Lock) gehört
    ''' Verglichen wird Tabellenname und RecordId
    ''' </summary>
    ''' <param name="row">Datensatz</param>
    ''' <returns>True/False</returns>
    Overloads Shared Function CompareDataRowWithLockInfo(row As DataRow) As Boolean
        If m_Lock Is Nothing Then Return False
        If row Is Nothing Then Return False
        Return Compare(row, m_Lock)
    End Function


#Region "Helper Functions"

    ''' <summary>
    ''' Vergleicht den Datensatz mit den LockInformationen
    ''' Tabellenname und RecordId
    ''' </summary>
    ''' <param name="row">Daternsatz</param>
    ''' <param name="pLockInfo">LockInformationen</param>
    ''' <returns>True/False</returns>
    Private Shared Function Compare(row As DataRow, pLockInfo As LockInfo) As Boolean
        ' Prüfe, ob DataRow auch zu den LockInfos gehören
        ' Tabellennamen vergleichen
        If pLockInfo.TableName.ToUpper() <> row.Table.TableName.ToUpper() Then
            MsgBox("LockInformationen passen nicht zum Datensatz! Falsche Tabelle!", vbOKOnly, "Hinweis")
            Return False
        End If
        ' PrimaryKey Field ermitteln
        Dim m_RecordIdField As String = GetPrimaryKeyFieldName(row)
        If m_RecordIdField = "" Then
            MsgBox("Kein PrimaryKey Field im DataRow mit AutoIncrement gefunden!", vbOKOnly, "HINWEIS")
            Return False
        End If
        ' RecordId vergleichen
        If CInt(row(m_RecordIdField)) <> pLockInfo.RecordID Then
            MsgBox("LockInformationen passen nicht zum Datensatz! Falsche RecordId!", vbOKOnly, "Hinweis")
            Return False
        End If
        Return True
    End Function

    ''' <summary>
    ''' Erzeugt die LockManager Tabelle falls nicht vorhanden
    ''' </summary>
    ''' <returns>True/False</returns>
    Private Shared Function CheckAndCreateTable() As Boolean
        m_ErrMsg = ""
        ' Prüfe, ob LockManager Tabelle vorhanden
        If CheckTable() Then Return True
        ' Tabelle existiert nicht, daher anlegen
        ' UNIQUE KEY `TableName_RecordId` (`TableName`,`RecordId`) damit Einträge vom selben Datensatz nicht doppelt sein können
        Dim dbSQL As String = "CREATE TABLE LockManager ("
        dbSQL += "LockId int(11) NOT NULL AUTO_INCREMENT,"
        dbSQL += "SessionId varchar(50) NOT NULL DEFAULT '',"
        dbSQL += "TableName varchar(45) NOT NULL DEFAULT '',"
        dbSQL += "RecordId int(11) NOT NULL DEFAULT 0,"
        dbSQL += "UserName varchar(45) NOT NULL DEFAULT '',"
        dbSQL += "ComputerName varchar(45) NOT NULL DEFAULT '',"
        dbSQL += "LockTimeout datetime NOT NULL DEFAULT CURRENT_TIMESTAMP, "
        dbSQL += "PRIMARY KEY (LockId), "
        dbSQL += "UNIQUE KEY `TableName_RecordId` (`TableName`,`RecordId`)"
        dbSQL += ") ENGINE=MyISAM AUTO_INCREMENT=1 DEFAULT CHARSET=utf8;"
        Try
            If Connection.State <> ConnectionState.Closed Then
                Connection.Close()
            End If
            Connection.Open()
            Using cmd As MySql.Data.MySqlClient.MySqlCommand = New MySql.Data.MySqlClient.MySqlCommand(dbSQL, Connection)
                cmd.ExecuteNonQuery()
            End Using
            Connection.Close()
            ' Prüfe nochmals, ob LockManager Tabelle nach dem Erzeugen jetzt vorhanden ist
            If CheckTable() Then
                Return True
            Else
                Return False
            End If
        Catch ex As Exception
            Connection.Close()
            m_ErrMsg = ex.Message
            Return False
        End Try
    End Function

    ''' <summary>
    ''' Prüft, ob die LockManager Tabelle vorhanden ist
    ''' </summary>
    ''' <returns>True/False</returns>
    Private Shared Function CheckTable() As Boolean
        Dim exists As String = ""
        Dim dbSQL As String = "SELECT IF((Select count(*) from information_schema.tables where table_name='LockManager')>0, 'Y','N')"
        m_ErrMsg = ""
        ' Prüft, ob die Tabelle "LockManager" vorhanden ist
        Try
            If Connection.State <> ConnectionState.Closed Then
                Connection.Close()
            End If
            Connection.Open()
            Using cmd As MySql.Data.MySqlClient.MySqlCommand = New MySql.Data.MySqlClient.MySqlCommand(dbSQL, Connection)
                exists = cmd.ExecuteScalar().ToString()
            End Using
            Connection.Close()
            If exists = "Y" Then Return True
            Return False
        Catch ex As Exception
            Connection.Close()
            m_ErrMsg = ex.Message
            Return False
        End Try
    End Function

    ''' <summary>
    ''' Veruscht, den Datensatz zu sperren bzw. aktualisieren
    ''' Wenn der Datensatz von einem anderen Benutzer gesperrt ist, wird die Property LockedInfo befüllt
    ''' </summary>
    ''' <returns>Treu/False</returns>
    Private Shared Function TryLock() As Boolean
        Dim dbTable As DataTable = New DataTable
        Dim dbSQL As String = "select * from LockManager where TableName='" + m_Lock.TableName + "' and "
        dbSQL += "RecordId=" + m_Lock.RecordID.ToString()
        If Connection.State <> ConnectionState.Closed Then
            Connection.Close()
        End If
        Connection.Open()
        ' Datensatz laden
        Using myCommand As MySql.Data.MySqlClient.MySqlCommand = New MySql.Data.MySqlClient.MySqlCommand(dbSQL, Connection)
            Using myReader As MySql.Data.MySqlClient.MySqlDataReader = myCommand.ExecuteReader()
                Try
                    If myReader.HasRows Then
                        dbTable.GetData(myReader, True)
                    End If
                Catch ex As Exception
                    m_ErrMsg = ex.Message
                    Return False
                End Try
            End Using
        End Using
        Connection.Close()
        ' Locking einmal auf False setzen
        m_Lock.Success = False
        ' Wurde ein Datensatz gefunden?
        If dbTable.Rows.Count <> 0 Then
            ' Jetzt noch prüfen, ob SessionId unterschiedlich sind
            If dbTable.Rows(0).Item("SessionId").ToString <> m_Lock.SessionId Then
                ' Ein anderer Benutzer hat den Datasatz gesperrt
                m_LockedInfo = New LockInfo()
                m_LockedInfo.SessionId = ""
                m_LockedInfo.TableName = dbTable.Rows(0).Item("TableName").ToString
                m_LockedInfo.RecordID = CInt(dbTable.Rows(0).Item("RecordId"))
                m_LockedInfo.UserName = dbTable.Rows(0).Item("UserName").ToString
                m_LockedInfo.ComputerName = dbTable.Rows(0).Item("ComputerName").ToString
                m_LockedInfo.LockTimeout = dbTable.Rows(0).Item("LockTimeout").ToString
                ' Jetzt prüfen, ob LockTimeout abgelaufen ist
                If CDate(GetServerDateTime()) > CDate(m_LockedInfo.LockTimeout) Then
                    ' Fremdes LockTimeout ist abgelaufen -> Sperren erlaubt
                    m_Lock.LockId = CInt(dbTable.Rows(0).Item("LockId"))
                    ' LockTimeout neu sezten und meine LockInfos speichern
                    If Not Update(False) Then
                        ' Da hat etwas nicht funktioniert
                        Return False
                    End If
                Else
                    ' Datensatzsperre ist noch gültig -> Neues Sperren nicht möglich
                    Return False
                End If
            Else
                ' ich bin es selbst
                m_Lock.LockId = CInt(dbTable.Rows(0).Item("LockId"))
                ' LockTimeOut neu setzen
                If Not Update() Then
                    ' Da hat etwas nicht funktioniert
                    Return False
                End If
            End If
        Else
            ' Keine Sperrung gefunden -> Datensatz kann gesperrt werden
            ' Meine LockInfos hinzufügen
            If Not Insert() Then
                ' Da hat etwas nicht funktioniert
                Return False
            End If
        End If
        m_Lock.Success = True
        Return True
    End Function

    ''' <summary>
    ''' Wird für die Prüfung benötigt, ob die Sperrung noch gültig ist.
    ''' Ist wichtig, falls das Timeout abgelaufen ist
    ''' d.h. vor dem Speichern des Datensatz nochmals LockManager.CheckLock() aufrufen
    ''' </summary>
    ''' <returns>True/False</returns>
    Private Shared Function TryCheckLock() As Boolean
        Dim dbTable As DataTable = New DataTable
        Dim dbSQL As String = "select * from LockManager where TableName='" + m_Lock.TableName + "' and "
        dbSQL += "RecordId=" + m_Lock.RecordID.ToString() + " and SessionId='" + m_Lock.SessionId + "'"
        If Connection.State <> ConnectionState.Closed Then
            Connection.Close()
        End If
        Connection.Open()
        ' Datensatz lesen
        Using myCommand As MySql.Data.MySqlClient.MySqlCommand = New MySql.Data.MySqlClient.MySqlCommand(dbSQL, Connection)
            Using myReader As MySql.Data.MySqlClient.MySqlDataReader = myCommand.ExecuteReader()
                Try
                    If myReader.HasRows Then
                        dbTable.GetData(myReader, True)
                    End If
                Catch ex As Exception
                    m_ErrMsg = ex.Message
                    Return False
                End Try
            End Using
        End Using
        Connection.Close()
        ' Locking einmal auf False setzen
        m_Lock.Success = False
        ' Wenn ein Datensatz gefunden wurde, dann ist meine SessionId diejenige, welcher den Datensatz gesperrt hat
        If dbTable.Rows.Count = 1 Then
            m_Lock.LockId = CInt(dbTable.Rows(0).Item("LockId"))
            ' Datensatzsperre aktualisieren
            If Not Update() Then
                ' Da ist etwas schief gegangen
                m_Lock.Success = False
                Return False
            End If
        Else
            ' Keine Sperrung gefunden -> Datensatzsperre nicht mehr aktuell,
            ' jemand anderer hat den Datensatz zwischenzeitlich gesperrt
            m_Lock.Success = False
            Return False
        End If
        ' Alles hat funktioniert, Locking noch gültig
        m_Lock.Success = True
        Return True
    End Function

    ''' <summary>
    ''' Liefert den Namen der Tabellenspalte, welches PrimaryKey und AutoIncrement ist
    ''' </summary>
    ''' <param name="row">zu sperrende Datensatz</param>
    ''' <returns>Name der Tabellenspalte</returns>
    Private Shared Function GetPrimaryKeyFieldName(row As DataRow) As String
        Try
            Dim retValue As String = ""
            Dim cols As DataColumn() = row.Table.PrimaryKey
            For Each col In cols
                If col.AutoIncrement Then
                    retValue = col.ColumnName
                    Exit For
                End If
            Next
            Return retValue
        Catch ex As Exception
            m_ErrMsg = ex.Message
            Return ""
        End Try
    End Function

#End Region

#Region "LockManager Tabelle - INSERT / UPDATE / DELETE"

    Private Shared dbSQL As String
    Private Shared affectRows As Integer

    ''' <summary>
    ''' Neue Datensatzsperre in die LockManager Tabelle eintragen
    ''' </summary>
    ''' <returns>True/False</returns>
    Private Shared Function Insert() As Boolean
        m_ErrMsg = ""
        dbSQL = "Insert into LockManager (SessionId, TableName, RecordId, UserName, ComputerName, LockTimeout) values("
        dbSQL += "'" + SessionId + "', "
        dbSQL += "'" + m_Lock.TableName + "', "
        dbSQL += m_Lock.RecordID.ToString() + ", "
        dbSQL += "'" + UserName + "', "
        dbSQL += "'" + m_Lock.ComputerName + "', "
        dbSQL += "'" + m_Lock.LockTimeout + "')"
        If Connection.State <> ConnectionState.Closed Then
            Connection.Close()
        End If
        Connection.Open()
        Try
            Using cmd As MySql.Data.MySqlClient.MySqlCommand = New MySql.Data.MySqlClient.MySqlCommand(dbSQL, Connection) With {.CommandTimeout = 5}
                affectRows = cmd.ExecuteNonQuery()
                affectRows = CInt(cmd.LastInsertedId)
                If affectRows > 0 Then
                    m_Lock.LockId = affectRows
                Else
                    m_ErrMsg = "Hinzufügen der Datensatzsperre nicht möglich!"
                    m_Lock.LockId = 0
                    Return False
                End If
            End Using
        Catch ex As Exception
            m_ErrMsg = ex.Message
            Connection.Close()
            Return False
        End Try
        Connection.Close()
        Return True
    End Function

    ''' <summary>
    ''' Update der Datensatzsperren in der LockManager Tabelle
    ''' </summary>
    ''' <param name="IsOwn">Flag, ob es noch immer der aktuell gesperrte Datensatz ist</param>
    ''' <returns>True/False</returns>
    Private Shared Function Update(Optional IsOwn As Boolean = True) As Boolean
        m_ErrMsg = ""
        dbSQL = "Update Lockmanager set "
        dbSQL += "SessionId = '" + SessionId + "', "
        dbSQL += "UserName = '" + UserName + "', "
        dbSQL += "ComputerName = '" + m_Lock.ComputerName + "', "
        dbSQL += "LockTimeout = '" + m_Lock.LockTimeout + "' "
        If IsOwn Then
            dbSQL += "where LockId = " + m_Lock.LockId.ToString() + " and SessionId = '" + SessionId + "'"
        Else
            dbSQL += "where RecordId = " + m_Lock.RecordID.ToString() + " and TableName = '" + m_Lock.TableName + "'"
        End If
        If Connection.State <> ConnectionState.Closed Then
            Connection.Close()
        End If
        Connection.Open()
        Try
            Using cmd As MySql.Data.MySqlClient.MySqlCommand = New MySql.Data.MySqlClient.MySqlCommand(dbSQL, Connection) With {.CommandTimeout = 5}
                affectRows = cmd.ExecuteNonQuery()
                If affectRows <= 0 Then
                    m_ErrMsg = "Update des gesperrten Datensatzen nicht möglich!"
                    Return False
                End If
            End Using
        Catch ex As Exception
            m_ErrMsg = ex.Message
            Connection.Close()
            Return False
        End Try
        Connection.Close()
        Return True
    End Function

    ''' <summary>
    ''' Löschen der Datensatzsperre aus der LockManager Tabelle
    ''' </summary>
    ''' <returns>True/False</returns>
    Private Shared Function Delete() As Boolean
        m_ErrMsg = ""
        dbSQL = "Delete from LockManager where LockId = " + m_Lock.LockId.ToString() + " and SessionId = '" + SessionId + "'"
        If Connection.State <> ConnectionState.Closed Then
            Connection.Close()
        End If
        Connection.Open()
        Try
            Using cmd As MySql.Data.MySqlClient.MySqlCommand = New MySql.Data.MySqlClient.MySqlCommand(dbSQL, Connection) With {.CommandTimeout = 5}
                affectRows = cmd.ExecuteNonQuery()
                If affectRows <= 0 Then
                    m_ErrMsg = "Kein gesperrter Datensatz gefunden!"
                    Return False
                End If
            End Using
        Catch ex As Exception
            m_ErrMsg = ex.Message
            Connection.Close()
            Return False
        End Try
        Connection.Close()
        Return True
    End Function

#End Region

End Class

